import Ogma from '@linkurious/ogma';
import startTutorial from './tutorial';
const FLAGS = {
  France: 'resources/flags/fr.svg',
  Japan: 'resources/flags/jp.svg',
  USA: 'resources/flags/us.svg',
  Brazil: 'resources/flags/br.svg'
};

const ogma = new Ogma({
  container: 'graph-container'
});
const openGroups = new Map();
let clickedGroup;
ogma.styles.setHoveredNodeAttributes({ outline: null });
ogma.styles.setSelectedNodeAttributes({ outline: null });
ogma.parse
  .jsonFromUrl('countries.json')
  .then(graph => {
    return ogma.addGraph(graph);
  })
  .then(() => {
    return ogma.layouts.force({ locate: true });
  });

// rule for regular nodes
ogma.styles.addNodeRule({
  innerStroke: { color: '#555' },
  image: {
    url: node => FLAGS[node.getData('location')],
    minVisibleSize: 0
  },
  opacity: node => (node.getData('open') ? 0.32 : undefined),
  badges: {
    bottomRight: { stroke: { color: '#999' } }
  }
});

// rule for groups
ogma.styles.addNodeRule(node => node.isVirtual(), {
  text: node => node.getData('country'),
  radius: 20,
  image: node => FLAGS[node.getData('country')],
  opacity: node => (openGroups.get(node.getId()) ? 0.5 : 1)
});

// adding grouping
const byCountry = ogma.transformations.addNodeGrouping({
  groupIdFunction: node => node.getData('location'),
  nodeGenerator: (nodes, country) => {
    return {
      id: 'special group ' + country,
      data: {
        country: country
      }
      // prefer not to generate attributes here
    };
  },
  // here we use a map to store which group is open or close.
  showContents: node => openGroups.get(node.getId()),
  // this callback will be called only when a new group is created
  onCreated: (metaNode, visible, subNodes, subEdges) => {
    if (visible) {
      return ogma.layouts.force({ nodes: subNodes });
    }
  },
  duration: 300,
  enabled: false
});

// this function will be called on each refresh.
function relayout() {
  if (clickedGroup) {
    // here we pin all the nodes but the one clicked,
    // so it will be the only one to move
    ogma.getNodes().setAttribute('layoutable', false);
    clickedGroup.setAttribute('layoutable', true);
    ogma.layouts.force();
    ogma.getNodes().setAttribute('layoutable', true);
    clickedGroup = null;
  } else {
    ogma.layouts.force();
  }
}

ogma.events.on('transformationEnabled', ({ target }) => {
  if (target !== byCountry) return;
  relayout();
});
ogma.events.on('transformationRefresh', ({ target }) => {
  if (target !== byCountry) return;
  relayout();
});
ogma.events.on('doubleclick', evt => {
  const target = evt.target;
  if (target && target.isNode && target.isVirtual()) {
    openGroups.set(target.getId(), !openGroups.get(target.getId()));
    clickedGroup = target;
    byCountry.refresh();
  }
});
startTutorial({ ogma, byCountry, ogma, openGroups });
