const continueButton = document.getElementById('continue');
let intro, stepIndex;

function open(ogma, byCountry, openGroups) {
  const closedNodes = ogma
    .getNodes()
    .filter(node => node.isVirtual() && !openGroups.get(node.getId()));
  closedNodes.forEach(node => openGroups.set(node.getId(), true));
  byCountry.refresh();
}
function close(ogma, byCountry, openGroups) {
  const openNodes = ogma
    .getNodes()
    .filter(node => node.isVirtual() && openGroups.get(node.getId()));
  openNodes.forEach(node => openGroups.set(node.getId(), false));
  byCountry.refresh();
}
function tutorialFactory({ byCountry, ogma, openGroups }) {
  let previousStepIndex = 0;
  const steps = [
    {
      title: 'Welcome',
      intro: 'Welcome to the Ogma groups tutorial!'
    },
    {
      title: 'Introduction',
      intro: 'You will find outlines in this interractive demo...'
    },
    {
      title: 'Introduction',
      intro: 'and more in-depth explaination below.'
    },
    {
      title: 'Introduction',
      intro: 'Groups are used to simplify your visualization.',
      onExit: () => {
        byCountry.enable(byCountry, openGroups);
      },
      onEnterBack: () => {
        byCountry.disable(byCountry, openGroups);
      }
    },
    {
      title: 'open and close',
      intro: 'You can open groups',
      onExit: () => open(ogma, byCountry, openGroups),
      onEnterBack: () => close(ogma, byCountry, openGroups)
    },
    {
      title: 'open and close',
      intro: 'and close them',
      onExit: () => close(ogma, byCountry, openGroups),
      onEnterBack: () => open(ogma, byCountry, openGroups)
    },
    {
      title: 'Good practices',
      intro: `The <b>groupIdFunction</b> decides in which group goes each node. 
    It is a good practice to use the <b>data</b> of the nodes in the grouping function<br/>
    <code>
      groupIdFunction: node => node.getData('location')
    </code>
    `
    },
    {
      title: 'Good practices',
      intro: `It is better not to generate grouped nodes with attributes,
    as they overwrite the attributes set by styleRules and classes.
    `
    },
    {
      title: 'Good practices',
      intro: `To layout your open groups, you should use 
    <code>
      ogma.events.on('transformationRefresh')
    </code>
    This way, you can make sure your layout is called after a node is open.
    `
    },
    {
      title: 'Good practices',
      intro: `You can pin nodes so only the node you are opening/closing moves.
    `
    },
    {
      title: 'Good practices',
      intro: `Try it: double click on close nodes.`,
      onExit: () => {
        continueButton.classList.remove('hidden');
        intro.exit();
      }
    },
    {
      title: 'Conclusion',
      intro: `You now master the good practices with grouping.
    You can modify this playground and try things on your own.`
    }
  ].map((step, i) => {
    const elt = document.createElement('p');
    document.body.appendChild(elt);
    elt.classList.add('dummy');
    elt.innerHTML = i;
    step.element = elt;
    step.position = 'top';
    return step;
  });
  continueButton.classList.add('hidden');

  return introJs()
    .setOptions({
      steps
    })
    .onchange(function (a) {
      stepIndex = +a.innerHTML;
      const step = steps[stepIndex];
      const previousStep = steps[previousStepIndex];
      let promise = Promise.resolve();
      // going
      if (previousStepIndex > stepIndex && step.onEnterBack) {
        promise = promise.then(() => step.onEnterBack());
      } else {
        // going forward
        if (previousStep && previousStep.onExit) {
          promise = promise.then(() => previousStep.onExit(step, previousStep));
        }
        if (step.onEnter) {
          promise = promise.then(() => step.onEnter(step, previousStep));
        }
      }
      previousStepIndex = stepIndex;
    });
}

export default function tutorial({ byCountry, ogma, openGroups }) {
  intro = tutorialFactory({ byCountry, ogma, openGroups }).start();
  continueButton.addEventListener('click', () => {
    const index = stepIndex + 1;
    intro = tutorialFactory({ ogma, byCountry, ogma, openGroups })
      .goToStepNumber(index)
      .start();
  });
}
